# Couches simplifiées des principales données de la base Admin Express IGN 2022
# ==============================================================================
# Géopackage WGS84
# 
# - Simplification topologique des couches IGN Admin Express France entière 
#   WGS84. 636 Mo (shp) -> 56 Mo (gpkg) ;
# - uniquement com, dep, reg, chl, epci ;
# - création de couches complémentaires "limites internes" (sans les côtes et 
#   frontières)
# - Défonçage de l'estuaire de la gironde (comparable à la version CARTO) pour
#   un meilleur rendu cartographique.
# - Ajout des surfaces cartésiennes (dans la projection locale) et géodésiques 
#   des données originales (=> les surfaces com. Gironde incluent l'estuaire 
#   alors que la géom. ne le comprend pas);
# - Ajout de Saint Barth et Saint Martin (champ du recensement agricole) ;
# - Chefs-lieux corrigés : complétés (+5) et replacés dans leur commune pour 
#   avoir une localisation ponctuelle plus précise qu'un centroïde quand 
#   possible ;
#   
# Requiert Nodejs et mapshaper :
# Windows :
# Installer https://nodejs.org/en/
# npm config set proxy "http://utilisateur_agricoll:*****@rie.proxy.national.agri:8080"
# npm install -g mapshaper
# ajouter si besoin C:\Users\xxxxxxxx\AppData\Roaming\npm au $PATH
# 
# Debian :
# apt install npm
# npm install -g mapshaper
# 
# michael.delorme - 2023-03-17 15:00:30


# paramètres --------------------------------------------------------------

library(tidyverse)
library(sf)
library(rmapshaper)
library(geojsonio)
library(janitor)
library(fs)
library(units)
library(glue)
# + installer {lwgeom}

# ADMIN-EXPRESS-COG édition 2022 France entière
# https://geoservices.ign.fr/adminexpress
# (en projection WGS84)
# si besoin dispo aussi sur :
# http://files.opendatarchives.fr/professionnels.ign.fr/adminexpress/
# 
# https://wxs.ign.fr/x02uy2aiwjo9bm8ce5plwqmr/telechargement/prepackage/ADMINEXPRESS-COG_SHP_WGS84G_PACK_2022-04-15$ADMIN-EXPRESS-COG_3-1__SHP_WGS84G_FRA_2022-04-15/file/ADMIN-EXPRESS-COG_3-1__SHP_WGS84G_FRA_2022-04-15.7z
source_adminexpress <- "~/data/adminexpress/ADMIN-EXPRESS-COG_3-1__SHP__FRA_WM_2022-04-15/ADMIN-EXPRESS-COG/1_DONNEES_LIVRAISON_2022-04-15/ADECOG_3-1_SHP_WGS84G_FRA"

# ADMIN-EXPRESS-COG-CARTO édition 2022 France entière (pour la Gironde)
# https://geoservices.ign.fr/adminexpress#telechargementCogCarto
# 
# https://wxs.ign.fr/x02uy2aiwjo9bm8ce5plwqmr/telechargement/prepackage/ADMINEXPRESS-COG-CARTO_SHP_WGS84G_PACK_2022-04-15$ADMIN-EXPRESS-COG-CARTO_3-1__SHP_WGS84G_FRA_2022-04-15/file/ADMIN-EXPRESS-COG-CARTO_3-1__SHP_WGS84G_FRA_2022-04-15.7z
source_adminexpress_carto <- "~/data/adminexpress/ADMIN-EXPRESS-COG-CARTO_3-1__SHP__FRA_WM_2022-04-15/ADMIN-EXPRESS-COG-CARTO/1_DONNEES_LIVRAISON_2022-04-15/ADECOGC_3-1_SHP_WGS84G_FRA"

# Communes BDTOPO (pour StB/StM, absent d'Admin Express)
# https://wxs.ign.fr/859x8t863h6a09o9o6fy4v60/telechargement/inspire/BDTOPO-FRANCE-ADMIN-SSDOUBLON-PACK_11.1$BDTOPO_2-0_ADMINISTRATIF_SHP_WGS84G_FRA_2011-01-01/file/BDTOPO_2-0_ADMINISTRATIF_SHP_WGS84G_FRA_2011-01-01.7z
# 
# voir similaire :
# http://files.opendatarchives.fr/professionnels.ign.fr/bdtopo/BDTOPO_3-0_2022-06-15/
source_bdtopo_977 <- "~/data/bdtopo/BDTOPO_3-0_TOUSTHEMES_GPKG_RGAF09UTM20_D977_2022-06-15/BDTOPO/1_DONNEES_LIVRAISON_2022-06-00173/BDT_3-0_GPKG_RGAF09UTM20_D977-ED2022-06-15/BDT_3-0_GPKG_RGAF09UTM20_D977-ED2022-06-15.gpkg"
source_bdtopo_978 <- "~/data/bdtopo/BDTOPO_3-0_TOUSTHEMES_GPKG_RGAF09UTM20_D978_2022-06-15/BDTOPO/1_DONNEES_LIVRAISON_2022-06-00173/BDT_3-0_GPKG_RGAF09UTM20_D978-ED2022-06-15/BDT_3-0_GPKG_RGAF09UTM20_D978-ED2022-06-15.gpkg"

# date de la donnée IGN
date_ign <- str_extract(source_adminexpress, "(?<=_)\\d{4}-\\d{2}-\\d{2}(?=/)")
annee_ign <- year(as.Date(date_ign))

# fichier en sortie
destination <- glue("~/data/adminexpress/adminexpress_cog_simpl_000_{annee_ign}.gpkg")


# originaux ---------------------------------------------------------------

# ouverture des fichiers SIG
list("COMMUNE", "DEPARTEMENT", "REGION", "EPCI", "CHFLIEU_COMMUNE") %>% 
  set_names(map(., ~ str_to_lower(if_else(str_length(.x) > 4, 
                                          str_sub(.x, 1, 3), 
                                          .x)))) %>%
  map(~ read_sf(source_adminexpress, layer = .x) %>%
          clean_names()) %>% 
  list2env(globalenv())

# ajout de stB et stM qu'on rattache virtuellement à la Guadeloupe
st_barth_st_martin <- bind_rows(
    source_bdtopo_977 %>% 
      read_sf(layer = "collectivite_territoriale"),
    source_bdtopo_978 %>% 
      read_sf(layer = "collectivite_territoriale")) %>% 
  clean_names() %>% 
  rename(geometry = geometrie) %>% 
  mutate(insee_com = glue("{code_insee}01"),
         insee_reg = "01_SBSM",
         statut = "Collectivité territoriale",
         nom = str_replace(nom_officiel, "Collectivité de ", ""),
         population = case_when(code_insee == "977" ~ 10585, # 2020
                                code_insee == "978" ~ 32358, # 2020
                                TRUE ~ NA_integer_)) %>% 
  select(id = cleabs, nom, insee_com, 
         statut , insee_dep = code_insee, insee_reg, population) %>% 
  st_transform("EPSG:4326")

com_complet <- com %>% 
  bind_rows(st_barth_st_martin)

reg <- reg %>% 
  add_row(insee_reg = "01_SBSM",
          nom = "Guadeloupe (Saint-Barthélemy/Saint-Martin)")

# ajouts des chefs-lieux manquants
# (villages détruits, StBStM)
chf_complet <- chf %>%
  bind_rows(com_complet %>% 
              anti_join(st_drop_geometry(chf),
                        by = c("id" = "id_com")) %>%
              st_point_on_surface() %>%
              select(id_com = id, nom))

# Extraction de la Gironde pour l'enlever par la suite (après le calcul de surface)
gironde <- dep %>% 
  filter(insee_dep %in% c("17", "33")) %>% 
  summarise() %>% 
  st_difference(read_sf(source_adminexpress_carto, 
                        layer = "DEPARTEMENT") %>%
                  clean_names() %>% 
                  filter(insee_dep %in% c("17", "33")) %>% 
                  summarise()) %>% 
  st_cast("POLYGON") %>% 
  mutate(surf = st_area(geometry)) %>% 
  slice_max(surf) %>% 
  select(-surf)


# simplifications ---------------------------------------------------------

check_sys_mapshaper()

# 6 min
# séparation en groupes de projection pour pouvoir re-projeter lors du calcul de
# surface
# utilisation de geojson_json pour éviter pb d'encodage
com_geojson <- com_complet %>%
  select(-nom_m) %>% 
  mutate(projection = case_when(
    insee_reg %in% c("01", "01_SBSM", "02") ~ "EPSG:5490", 
                          insee_reg == "03" ~ "EPSG:2972",
                          insee_reg == "04" ~ "EPSG:2975", 
                          insee_reg == "06" ~ "EPSG:4471",
                                       TRUE ~ "EPSG:2154")) %>% 
  group_by(projection) %>% 
  group_modify(
    ~ mutate(.x,
       surf_adminexpress_geo_ha = geometry %>% 
         st_area() %>% 
         set_units(ha) %>% 
         round() %>% 
         as.integer(),
       surf_adminexpress_prj_ha = geometry %>% 
         st_transform(pull(.y, projection)) %>% 
         st_area() %>% 
         set_units(ha) %>% 
         round() %>% 
         as.integer())) %>%
  ungroup() %>% 
  select(-projection) %>% 
  st_as_sf() %>% 
  st_difference(gironde) %>% 
  geojson_json(group = "insee_com", precision = 6) 

# Simplification et agrégations aux échelons supérieurs
# avec 2 % on est un peu plus détaillé que GéoFLA
com_simpl <- com_geojson %>%
  ms_simplify(keep = 0.02,
              method = "vis",
              weighting = 0.7,
              keep_shapes = TRUE,
              sys = TRUE)

# pour recentrer les chefs-lieux
com_simpl_sf <- com_simpl %>% 
  geojson_sf() %>% 
  as_tibble() %>% 
  st_sf() 


# simplification des autres niveaux hiérarchiques à partir des communes
dep_simpl <- com_simpl %>% 
  ms_dissolve(field = "insee_dep", sys = TRUE)

reg_simpl <- com_simpl %>% 
  ms_dissolve(field = "insee_reg", sys = TRUE)

epci_simpl <- com_simpl %>% 
  ms_dissolve(field = "siren_epci", sys = TRUE)


# vérifications -----------------------------------------------------------

epci_simpl %>% 
  geojson_sf() %>% 
  mutate(siren_epci = str_remove(siren_epci, "200054781/")) %>% # suppression des EPCI doubles Grand Paris
  anti_join(st_drop_geometry(epci), by = c("siren_epci" = "code_siren"))

epci %>%
  anti_join(epci_simpl %>% 
              geojson_sf() %>% 
              mutate(siren_epci = str_remove(siren_epci, "200054781/")) %>% 
              st_drop_geometry(), by = c("code_siren" = "siren_epci"))


# certaines communes ont leur chef lieu dans une autre commune
# life is weird d'une part (15) et d'autre part les simplifications ont généré
# des erreurs (env. 53)
externes <- chf_complet %>%
  select(-id) %>% 
  st_join(com_simpl_sf %>% 
            select(id, nom_com = nom)) %>%
  filter(id_com != id)

# on crée un point dans la vraie commune...
chf_virtuel <- com_simpl_sf %>%
  filter(id %in% externes$id_com) %>%
  select(id_com = id, nom) %>%
  st_point_on_surface()

# ...et on remplace
chf_nettoye <- chf_complet %>%
  filter(!id_com %in% externes$id_com) %>%
  bind_rows(chf_virtuel) %>%
  left_join(com_simpl_sf %>%
              st_drop_geometry() %>%
              select(id, insee_com, nom_com = nom, insee_dep, insee_reg, population),
            by = c("id_com" = "id")) %>%
  mutate(nom_chf = coalesce(nom, nom_com)) %>%
  select(insee_com, nom_com, insee_dep, insee_reg, population, nom_chf, id_com) %>%
  arrange(insee_com)


# Ajouts attributs et export ----------------------------------------------

com_simpl_sf %>% 
  select(-rmapshaperid) %>% 
  st_write(destination, 
           layer = "commune",
           layer_options = c(glue("IDENTIFIER=Communes Adminexpress {annee_ign} simplifiées"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper, ajout StBarth/StMartin, suppr. estuaire Gironde.")),
           delete_layer = TRUE)

dep_simpl %>% 
  geojson_sf() %>% 
  select(-rmapshaperid) %>% 
  left_join(dep %>% 
              st_drop_geometry() %>% 
              select(-nom_m),
            by = "insee_dep") %>% 
  mutate(insee_reg = if_else(insee_dep %in% c("977", "978"), "01_SMSB", insee_reg)) %>% 
  st_write(destination, 
           layer = "departement",
           layer_options = c(glue("IDENTIFIER=Départements Adminexpress {annee_ign} simplifiés"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper, ajout StBarth/StMartin, suppr. estuaire Gironde.")),
           delete_layer = TRUE)

reg_simpl %>% 
  geojson_sf() %>% 
  select(-rmapshaperid) %>% 
  left_join(reg %>% 
              st_drop_geometry() %>% 
              select(-nom_m), 
            by = "insee_reg") %>% 
  st_write(destination,
           layer = "region",
           layer_options = c(glue("IDENTIFIER=Régions Adminexpress {annee_ign} simplifiées"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper, ajout StBarth/StMartin, suppr. estuaire Gironde.")),
           delete_layer = TRUE)

chf_nettoye %>% 
  st_write(destination, 
           layer = "cheflieu",
           layer_options = c(glue("IDENTIFIER=chefs-lieux Adminexpress {annee_ign} corrigés"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Ajout communes mortes, StBarth/StMartin, replace dans la commune.")),
           delete_layer = TRUE)

# cas particulier du grand paris qui est un epci d'epci
gp <- epci_simpl %>% 
  geojson_sf() %>% 
  filter(str_detect(siren_epci, "^200054781")) %>% 
  mutate(siren_epci = "200054781") %>% 
  group_by(siren_epci) %>% 
  summarise()

epci_simpl %>% 
  geojson_sf() %>% 
  select(-rmapshaperid) %>% 
  mutate(siren_epci = str_remove(siren_epci, "200054781/")) %>% # suppression des codes EPCI doubles Grand Paris
  filter(siren_epci != "200054781",
         siren_epci != "NR",
         !is.na(siren_epci)) %>%
  bind_rows(gp) %>% 
  left_join(st_drop_geometry(epci), by = c("siren_epci" = "code_siren")) %>% 
  st_cast("MULTIPOLYGON") %>% 
  st_write(destination, layer = "epci",
           layer_options = c(glue("IDENTIFIER=EPCI Adminexpress {annee_ign} simplifiés"),
                             glue("DESCRIPTION=Établissement public de coopération intercommunale France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper, suppr. estuaire Gironde.")),
           delete_layer = TRUE)

# uniquement les frontières internes
com_simpl %>%
  ms_innerlines(sys = TRUE) %>%
  geojson_sf() %>%
  select(-rmapshaperid) %>% 
  st_write(destination,
           layer = "commune_int",
           layer_options = c(glue("IDENTIFIER=Frontières internes des communes Adminexpress {annee_ign} simplifiées"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper.
                                  Extraction des frontières internes.")),
           delete_layer = TRUE)

dep_simpl %>%
  ms_innerlines(sys = TRUE) %>%
  geojson_sf() %>%
  select(-rmapshaperid) %>% 
  st_write(destination,
           layer = "departement_int",
           layer_options = c(glue("IDENTIFIER=Frontières internes des départements Adminexpress {annee_ign} simplifiés"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper.
                                  Extraction des frontières internes.")),
           delete_layer = TRUE)

reg_simpl %>%
  ms_innerlines(sys = TRUE) %>%
  geojson_sf() %>%
  select(-rmapshaperid) %>% 
  st_write(destination,
           layer = "region_int",
           layer_options = c(glue("IDENTIFIER=Frontières internes des régions Adminexpress {annee_ign} simplifiées"),
                             glue("DESCRIPTION=France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper.
                                  Extraction des frontières internes.")),
           delete_layer = TRUE)

epci_simpl %>%
  ms_innerlines(sys = TRUE) %>%
  geojson_sf() %>%
  select(-rmapshaperid) %>% 
  st_write(destination,
           layer = "epci_int",
           layer_options = c(glue("IDENTIFIER=Frontières internes des EPCI Adminexpress {annee_ign} simplifiés"),
                             glue("DESCRIPTION=Établissement public de coopération intercommunale France WGS84 version COG ({date_ign}).
                                  Simplification mapshaper.
                                  Extraction des frontières internes.")),
           delete_layer = TRUE)
